﻿// Photoshopのレイヤーをレイヤー毎にファイルに書き出す
#target "Photoshop"
// 階層状態のレイヤーをフラットな状態にする
function flatLayer(docObj){
    for(var i=0; i<docObj.artLayers.length; i++){
        // レイヤーを先頭に移動させる。背景の場合は移動できないので無視する
        try{
            docObj.artLayers[i].move(activeDocument.layers[0], ElementPlacement.PLACEBEFORE);
        }catch(e){}
    }
    // レイヤーセットは再帰を使って処理
    for(var i=0; i<docObj.layerSets.length; i++){
        flatLayer(docObj.layerSets[i]);
    }
}
// artLayerの数を調べる。最初の階層以外に存在するならばtrueを返す
function checkLayer(docObj, level){
    if((level > 0) && (docObj.artLayers.length > 0)){ return true; }
    // レイヤーセットは再帰を使って処理
    for(var i=0; i<docObj.layerSets.length; i++){
        level = level + 1;
        return checkLayer(docObj.layerSets[i], level);
    }
    return false;
}
// 一番上のレイヤー以外のレイヤーを全て削除する
function deleteLayer(){
    for(var i=activeDocument.layers.length-1; i>0; i--){
        try{
            activeDocument.layers[i].remove();
        }catch(e){}
    }
}
// メイン処理
(function(){
    // 保存先のフォルダを選択する
    var baseFolder = Folder.selectDialog("保存先のフォルダを指定してください");
    if (!baseFolder){ return; }
    // ドキュメントを複製する
    var baseDoc = app.activeDocument.duplicate(app.activeDocument);
    // レイヤーをフラットな状態にする
    while(checkLayer(baseDoc, 0)){
        flatLayer(baseDoc);
    }
    // レイヤーセットを全て削除する（不要になったため）
    for(var i=baseDoc.layerSets.length-1; i>=0; i--){
        baseDoc.layerSets[i].remove();
    }
    // レイヤーを書き出す
    for(var i=0; i<baseDoc.artLayers.length; i++){
        var saveName = baseFolder.fsName+"/"+baseDoc.artLayers[i].name+".psd";
        var dupDoc = app.activeDocument.duplicate(app.activeDocument);
        // 保存すべきレイヤー以外を削除する
        for(var j=0; j<dupDoc.artLayers.length; j++){
            if (i == j){
                // 保存用に複製する
                try {   // 背景の場合は移動できないのでエラーになる。このエラーは無視する
                    // レイヤーを先頭に移動させる。背景の場合は移動できないので無視する
                    dupDoc.artLayers[i].move(activeDocument.layers[0], ElementPlacement.PLACEBEFORE);
                }catch(e){}
                // 先頭のレイヤー以外を削除する
                deleteLayer();
                break;
            }
        }
        // PSD形式で保存し不要になったドキュメントを閉じる
        var fileObj = new File(saveName);
        activeDocument.saveAs(fileObj);
        activeDocument.close(SaveOptions.DONOTSAVECHANGES);
    }
    // 複製したドキュメントを保存せず閉じる
    activeDocument.close(SaveOptions.DONOTSAVECHANGES);
})();
